@@ -3,69 +3,61 @@ from __future__ import absolute_import
 # flake8: noqa
 
 # import apis into api package
+from kubernetes.client.api.well_known_api import WellKnownApi
 from kubernetes.client.api.admissionregistration_api import AdmissionregistrationApi
 from kubernetes.client.api.admissionregistration_v1_api import AdmissionregistrationV1Api
-from kubernetes.client.api.admissionregistration_v1beta1_api import AdmissionregistrationV1beta1Api
 from kubernetes.client.api.apiextensions_api import ApiextensionsApi
 from kubernetes.client.api.apiextensions_v1_api import ApiextensionsV1Api
-from kubernetes.client.api.apiextensions_v1beta1_api import ApiextensionsV1beta1Api
 from kubernetes.client.api.apiregistration_api import ApiregistrationApi
 from kubernetes.client.api.apiregistration_v1_api import ApiregistrationV1Api
-from kubernetes.client.api.apiregistration_v1beta1_api import ApiregistrationV1beta1Api
 from kubernetes.client.api.apis_api import ApisApi
 from kubernetes.client.api.apps_api import AppsApi
 from kubernetes.client.api.apps_v1_api import AppsV1Api
-from kubernetes.client.api.auditregistration_api import AuditregistrationApi
-from kubernetes.client.api.auditregistration_v1alpha1_api import AuditregistrationV1alpha1Api
 from kubernetes.client.api.authentication_api import AuthenticationApi
 from kubernetes.client.api.authentication_v1_api import AuthenticationV1Api
-from kubernetes.client.api.authentication_v1beta1_api import AuthenticationV1beta1Api
 from kubernetes.client.api.authorization_api import AuthorizationApi
 from kubernetes.client.api.authorization_v1_api import AuthorizationV1Api
-from kubernetes.client.api.authorization_v1beta1_api import AuthorizationV1beta1Api
 from kubernetes.client.api.autoscaling_api import AutoscalingApi
 from kubernetes.client.api.autoscaling_v1_api import AutoscalingV1Api
+from kubernetes.client.api.autoscaling_v2_api import AutoscalingV2Api
 from kubernetes.client.api.autoscaling_v2beta1_api import AutoscalingV2beta1Api
 from kubernetes.client.api.autoscaling_v2beta2_api import AutoscalingV2beta2Api
 from kubernetes.client.api.batch_api import BatchApi
 from kubernetes.client.api.batch_v1_api import BatchV1Api
 from kubernetes.client.api.batch_v1beta1_api import BatchV1beta1Api
-from kubernetes.client.api.batch_v2alpha1_api import BatchV2alpha1Api
 from kubernetes.client.api.certificates_api import CertificatesApi
-from kubernetes.client.api.certificates_v1beta1_api import CertificatesV1beta1Api
+from kubernetes.client.api.certificates_v1_api import CertificatesV1Api
 from kubernetes.client.api.coordination_api import CoordinationApi
 from kubernetes.client.api.coordination_v1_api import CoordinationV1Api
-from kubernetes.client.api.coordination_v1beta1_api import CoordinationV1beta1Api
 from kubernetes.client.api.core_api import CoreApi
 from kubernetes.client.api.core_v1_api import CoreV1Api
 from kubernetes.client.api.custom_objects_api import CustomObjectsApi
 from kubernetes.client.api.discovery_api import DiscoveryApi
+from kubernetes.client.api.discovery_v1_api import DiscoveryV1Api
 from kubernetes.client.api.discovery_v1beta1_api import DiscoveryV1beta1Api
 from kubernetes.client.api.events_api import EventsApi
+from kubernetes.client.api.events_v1_api import EventsV1Api
 from kubernetes.client.api.events_v1beta1_api import EventsV1beta1Api
-from kubernetes.client.api.extensions_api import ExtensionsApi
-from kubernetes.client.api.extensions_v1beta1_api import ExtensionsV1beta1Api
 from kubernetes.client.api.flowcontrol_apiserver_api import FlowcontrolApiserverApi
-from kubernetes.client.api.flowcontrol_apiserver_v1alpha1_api import FlowcontrolApiserverV1alpha1Api
+from kubernetes.client.api.flowcontrol_apiserver_v1beta1_api import FlowcontrolApiserverV1beta1Api
+from kubernetes.client.api.flowcontrol_apiserver_v1beta2_api import FlowcontrolApiserverV1beta2Api
+from kubernetes.client.api.internal_apiserver_api import InternalApiserverApi
+from kubernetes.client.api.internal_apiserver_v1alpha1_api import InternalApiserverV1alpha1Api
 from kubernetes.client.api.logs_api import LogsApi
 from kubernetes.client.api.networking_api import NetworkingApi
 from kubernetes.client.api.networking_v1_api import NetworkingV1Api
-from kubernetes.client.api.networking_v1beta1_api import NetworkingV1beta1Api
 from kubernetes.client.api.node_api import NodeApi
+from kubernetes.client.api.node_v1_api import NodeV1Api
 from kubernetes.client.api.node_v1alpha1_api import NodeV1alpha1Api
 from kubernetes.client.api.node_v1beta1_api import NodeV1beta1Api
+from kubernetes.client.api.openid_api import OpenidApi
 from kubernetes.client.api.policy_api import PolicyApi
+from kubernetes.client.api.policy_v1_api import PolicyV1Api
 from kubernetes.client.api.policy_v1beta1_api import PolicyV1beta1Api
 from kubernetes.client.api.rbac_authorization_api import RbacAuthorizationApi
 from kubernetes.client.api.rbac_authorization_v1_api import RbacAuthorizationV1Api
-from kubernetes.client.api.rbac_authorization_v1alpha1_api import RbacAuthorizationV1alpha1Api
-from kubernetes.client.api.rbac_authorization_v1beta1_api import RbacAuthorizationV1beta1Api
 from kubernetes.client.api.scheduling_api import SchedulingApi
 from kubernetes.client.api.scheduling_v1_api import SchedulingV1Api
-from kubernetes.client.api.scheduling_v1alpha1_api import SchedulingV1alpha1Api
-from kubernetes.client.api.scheduling_v1beta1_api import SchedulingV1beta1Api
-from kubernetes.client.api.settings_api import SettingsApi
-from kubernetes.client.api.settings_v1alpha1_api import SettingsV1alpha1Api
 from kubernetes.client.api.storage_api import StorageApi
 from kubernetes.client.api.storage_v1_api import StorageV1Api
 from kubernetes.client.api.storage_v1alpha1_api import StorageV1alpha1Api
